@@ -40,13 +40,13 @@ G_BEGIN_DECLS
 typedef struct RsvgSaxHandler RsvgSaxHandler;
 typedef struct RsvgDrawingCtx RsvgDrawingCtx;
 typedef struct RsvgRender RsvgRender;
-typedef struct RsvgDimensionData RsvgDimensionData;
 typedef struct _RsvgPropertyBag RsvgPropertyBag;
 typedef struct _RsvgState RsvgState;
 typedef struct _RsvgDefs RsvgDefs;
 typedef struct _RsvgNode RsvgNode;
 typedef struct _RsvgFilter RsvgFilter;
 typedef struct _RsvgNodeChars RsvgNodeChars;
+typedef struct _RsvgIRect RsvgIRect;
 
 /* prepare for gettext */
 #ifndef _
@@ -65,7 +65,6 @@ typedef struct _RsvgNodeChars RsvgNodeChars;
 #  endif /* G_PI */
 #endif /*  M_PI  */
 
-
 struct RsvgSaxHandler {
 	void (*free) (RsvgSaxHandler *self);
 	void (*start_element) (RsvgSaxHandler *self, const xmlChar *name, RsvgPropertyBag *atts);
@@ -122,6 +121,10 @@ struct RsvgHandle {
 	void * gzipped_data; /* really a GsfOutput */
 };
 
+typedef struct{
+	double w, h;
+} RsvgViewBox; 
+
 /*Contextual information for the drawing phase*/
 
 struct RsvgDrawingCtx {
@@ -132,8 +135,9 @@ struct RsvgDrawingCtx {
 	gchar * base_uri;
 	GMemChunk * state_allocator;
 	PangoContext *pango_context;
-	double dpi_x;
-	double dpi_y;
+	double dpi_x, dpi_y;
+	RsvgViewBox vb;
+	GSList * vb_stack;
 };
 
 /*Abstract base class for context for our backends (one as yet)*/
@@ -148,14 +152,26 @@ struct RsvgRender {
 	void (* push_discrete_layer) (RsvgDrawingCtx *ctx);
 	void (* add_clipping_rect) (RsvgDrawingCtx *ctx,
 								double x, double y, double w, double h);
+	GdkPixbuf * (* get_image_of_node) (RsvgDrawingCtx *ctx, RsvgNode * drawable,
+									   double w, double h);
 };
 
-struct RsvgDimensionData {
-	int width;
-	int height;
-	gdouble em, ex;
+
+typedef struct{
+	double length;
+	char factor;
+} RsvgLength;
+
+struct _RsvgIRect {
+	int x0, y0, x1, y1;
 };
 
+typedef struct {
+	gdouble x, y, w, h;
+	gboolean virgin;
+	double affine[6];
+} RsvgBbox;
+
 typedef enum {
 	RSVG_SIZE_ZOOM,
 	RSVG_SIZE_WH,
@@ -263,6 +279,8 @@ void rsvg_render_image (RsvgDrawingCtx *ctx, GdkPixbuf * pb,
 void rsvg_render_free (RsvgRender * render);
 void rsvg_add_clipping_rect (RsvgDrawingCtx *ctx, double x, double y, 
 							 double w, double h);
+GdkPixbuf * rsvg_get_image_of_node(RsvgDrawingCtx *ctx, RsvgNode * drawable,
+								   double w, double h);
 
 
 void
@@ -316,6 +334,26 @@ _rsvg_affine_equal (double matrix1[6], double matrix2[6]);
 void
 rsvg_node_set_atts(RsvgNode * node, RsvgHandle * ctx, RsvgPropertyBag * atts);
 
+void
+rsvg_drawing_ctx_free (RsvgDrawingCtx *handle);
+
+void rsvg_bbox_init(RsvgBbox * self, double * affine);
+void rsvg_bbox_insert(RsvgBbox * dst, RsvgBbox * src);
+void rsvg_bbox_clip(RsvgBbox * dst, RsvgBbox * src);
+
+double
+_rsvg_css_normalize_length_struct(const RsvgLength * in, RsvgDrawingCtx * ctx, 
+								  char dir);
+double
+_rsvg_css_hand_normalize_length_struct(const RsvgLength * in, gdouble pixels_per_inch,
+									   gdouble width_or_height, gdouble font_size);
+
+RsvgLength
+_rsvg_css_parse_length_struct(const char *str);
+
+void _rsvg_push_view_box(RsvgDrawingCtx *ctx, double w, double h);
+void _rsvg_pop_view_box(RsvgDrawingCtx *ctx);
+
 G_END_DECLS
 
 #endif
